"""A timer that invokes a specified callback periodically.
"""
# Author: Prabhu Ramachandran <prabhu@aero.iitb.ac.in>
# Copyright (c) 2006-2007,  Enthought, Inc.
# License: BSD Style.

# Import the toolkit specific version.
from pyface.toolkit import toolkit_object
Timer = toolkit_object('timer.timer:Timer')
