#------------------------------------------------------------------------------
# Copyright (c) 2005, Enthought, Inc.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#
# Author: Enthought, Inc.
# Description: <Enthought pyface package component>
#------------------------------------------------------------------------------
""" A tool bar manager realizes itself in a tool palette control. """


# Import the toolkit specific version.
from pyface.toolkit import toolkit_object
ToolPaletteManager = toolkit_object('action.tool_palette_manager:ToolPaletteManager')

### EOF #######################################################################
